@@ -17,4 +17,13 @@ utils::globalVariables(c("y", "year", "smooth_vals", "smooth_num",
                          "name", "needed", "nmix_trendmap", "orig_formula",
                          "trial", "use_var1", "use_var1cor", "xcols_drop",
                          "time_lag", "dis_time", "maxt", "orig_rows",
-                         "matches", "time.", "file_name", ".data"))
+                         "matches", "time.", "file_name", ".data",
+                         "horizon", "target", "Series", "evd", "mean_evd",
+                         "total_evd", "smooth_label", "by_variable",
+                         "gr", "tot_subgrs", "subgr", "lambda",
+                         "level", "sim_hilbert_gp", "trend_model",
+                         "jags_path", "x", "elpds", "pareto_ks",
+                         "value", "threshold", "colour", "resids",
+                         "c_dark", "eval_timepoints", "yqlow",
+                         "ymidlow", "ymidhigh", "yqhigh", "preds",
+                         "yhigh", "ylow", "weight", "orig_weight"))
