@@ -1,9 +1,9 @@
-#'Predict from the GAM component of an mvgam model
+#'Predict from a fitted \pkg{mvgam} model
 #'@importFrom stats predict
 #'@inheritParams brms::fitted.brmsfit
-#'@param object \code{list} object returned from \code{mvgam}. See [mvgam()]
-#'@param newdata Optional \code{dataframe} or \code{list} of test data containing the
-#'variables included in the linear predictor of \code{formula}. If not supplied,
+#'@param object \code{list} object of class \code{mvgam} or \code{jsdgam}. See [mvgam()]
+#'@param newdata Optional \code{dataframe} or \code{list} of test data containing the same variables
+#'that were included in the original `data` used to fit the model. If not supplied,
 #'predictions are generated for the original observations used for the model fit.
 #'@param data_test Deprecated. Still works in place of \code{newdata} but users are recommended to use
 #'\code{newdata} instead for more seamless integration into `R` workflows
@@ -75,7 +75,8 @@
 #'              family = gaussian(),
 #'              trend_model = AR(),
 #'              noncentred = TRUE,
-#'              chains = 2)
+#'              chains = 2,
+#'              silent = 2)
 #'
 #'# Generate predictions against observed data
 #'preds <- predict(mod1, summary = TRUE)
@@ -106,10 +107,16 @@ predict.mvgam = function(object,
 
   # newdata needs to have a 'series' indicator in it for integrating
   # over the trend uncertainties
-  if(!'series' %in% names(newdata)){
-    newdata$series <- factor(rep(levels(object$obs_data$series)[1],
-                                 length(newdata[[1]])),
-                             levels = levels(object$obs_data$series))
+  if(inherits(object, 'jsdgam')){
+    newdata <- validate_series_time(data = newdata,
+                                    trend_model = attr(object$model_data, 'prepped_trend_model'),
+                                    check_levels = FALSE,
+                                    check_times = FALSE)
+  } else {
+    newdata <- validate_series_time(data = newdata,
+                                    trend_model = object$trend_model,
+                                    check_levels = FALSE,
+                                    check_times = FALSE)
   }
 
   type <- match.arg(arg = type, choices = c("link",
@@ -151,8 +158,8 @@ predict.mvgam = function(object,
 
 
     # If a linear predictor was supplied for the latent process models, calculate
-    # predictions by assuming the trend is stationary (this is basically what brms)
-    # does when predicting for autocor() models
+    # predictions by assuming the trend is stationary (this is basically what brms
+    # does when predicting for autocor() models)
     if(!is.null(object$trend_call)){
 
       # Linear predictor matrix for the latent process models
@@ -162,7 +169,9 @@ predict.mvgam = function(object,
                             mgcv_model = object$trend_mgcv_model)
 
       # Extract process error estimates
-      if(attr(object$model_data, 'trend_model') %in% c('None', 'RW','AR1','AR2','AR3','CAR1')){
+      if(attr(object$model_data, 'trend_model') %in%
+         c('None', 'RW', 'AR1', 'AR2',
+           'AR3', 'CAR1', 'ZMVN')){
         if(object$family == 'nmix'){
           family_pars <- list(sigma_obs = .Machine$double.eps)
         } else {
@@ -213,21 +222,76 @@ predict.mvgam = function(object,
       })
       names(family_extracts) <- names(family_pars)
 
-      # Pre-multiply the linear predictors
-      all_linpreds <- as.matrix(as.vector(t(apply(as.matrix(betas), 1,
-                                                  function(row) Xp %*% row +
-                                                    attr(Xp, 'model.offset')))))
-      attr(all_linpreds, 'model.offset') <- 0
-
       # Trend stationary predictions
       if(!process_error){
         family_extracts <- list(sigma_obs = .Machine$double.eps)
       }
-      trend_predictions <- mvgam_predict(family = 'gaussian',
-                                         Xp = all_linpreds,
-                                         type = 'response',
-                                         betas = 1,
-                                         family_pars = family_extracts)
+      if(inherits(object, 'jsdgam')){
+        # JSDMs should generate one set of predictions per latent variable and then
+        # create a weighted set of predictions based on the loading estimates
+        lv_coefs <- mcmc_chains(object$model_output, 'lv_coefs')
+        n_draws <- dim(mcmc_chains(object$model_output, 'b'))[1]
+        series_ind <- as.numeric(newdata$series)
+
+        trend_predictions_raw <- lapply(1:object$n_lv, function(x){
+          # Linear predictor matrix for the latent process models
+          Xp <- trend_Xp_matrix(newdata = newdata,
+                                trend_map = data.frame(
+                                  trend = x,
+                                  series = unique(object$trend_map$series)
+                                  ),
+                                series = 'all',
+                                mgcv_model = object$trend_mgcv_model)
+          all_linpreds <- as.matrix(as.vector(t(apply(as.matrix(betas), 1,
+                                                      function(row) Xp %*% row +
+                                                        attr(Xp, 'model.offset')))))
+          attr(all_linpreds, 'model.offset') <- 0
+          pred_vec <- mvgam_predict(family = 'gaussian',
+                                    Xp = all_linpreds,
+                                    type = 'response',
+                                    betas = 1,
+                                    family_pars = family_extracts)
+          matrix(pred_vec, nrow = NROW(betas))
+        })
+
+        # Create weighted set of predictions using the loadings
+        weighted_mat = function(pred_matrices,
+                                weights,
+                                draw = 1,
+                                obs = 1){
+          lv_draws <- unlist(lapply(pred_matrices,
+                                    function(x) x[draw, obs]),
+                             use.names = FALSE)
+          as.vector(lv_draws %*% weights)
+        }
+
+        trend_predictions <- matrix(NA, nrow = n_draws,
+                                    ncol = length(newdata[[1]]))
+        n_lv <- object$n_lv
+        for(i in 1:n_draws){
+          for(x in 1:length(newdata[[1]])){
+            trend_predictions[i, x] <- weighted_mat(trend_predictions_raw,
+                                                    matrix(lv_coefs[i,],
+                                                           nrow = n_lv)[,series_ind[x]],
+                                                    draw = i,
+                                                    obs = x)
+          }
+        }
+        trend_predictions <- as.vector(trend_predictions)
+
+      } else {
+        # Pre-multiply the linear predictors
+        all_linpreds <- as.matrix(as.vector(t(apply(as.matrix(betas), 1,
+                                                    function(row) Xp %*% row +
+                                                      attr(Xp, 'model.offset')))))
+        attr(all_linpreds, 'model.offset') <- 0
+        trend_predictions <- mvgam_predict(family = 'gaussian',
+                                           Xp = all_linpreds,
+                                           type = 'response',
+                                           betas = 1,
+                                           family_pars = family_extracts)
+      }
+
 
     } else if(attr(object$model_data, 'trend_model') != 'None' & process_error){
       # If no linear predictor for the trends but a dynamic trend model was used,
@@ -269,7 +333,7 @@ predict.mvgam = function(object,
       if(!object$use_lv){
 
         if(attr(object$model_data, 'trend_model') %in%
-           c('RW','AR1','AR2','AR3','VAR1','CAR1')){
+           c('RW','AR1','AR2','AR3','VAR1','CAR1','ZMVN')){
           family_pars <- list(sigma_obs = mcmc_chains(object$model_output,
                                                       'sigma'))
         }
@@ -451,9 +515,9 @@ terms_preds = function(object, newdata, summary = TRUE,
                           series = 'all',
                           mgcv_model = object$trend_mgcv_model)
     betas <- mcmc_chains(object$model_output, 'b_trend')
-    effect_names <- colnames(predict(object$trend_mgcv_model,
-                                         type = 'terms',
-                                         se.fit = FALSE))
+    effect_names <- colnames(predict(relabel_gps(object$trend_mgcv_model),
+                                     type = 'terms',
+                                     se.fit = FALSE))
     effect_names <- gsub('series', 'trend',
                          effect_names, fixed = TRUE)
     coef_names <- names(coef(object$trend_mgcv_model))
@@ -463,9 +527,9 @@ terms_preds = function(object, newdata, summary = TRUE,
     Xp <- obs_Xp_matrix(newdata = newdata,
                         mgcv_model = object$mgcv_model)
     betas <- mcmc_chains(object$model_output, 'b')
-    effect_names <- colnames(predict(object$mgcv_model,
-                                         type = 'terms',
-                                         se.fit = FALSE))
+    effect_names <- colnames(predict(relabel_gps(object$mgcv_model),
+                                     type = 'terms',
+                                     se.fit = FALSE))
     coef_names <- names(coef(object$mgcv_model))
   }
 
