@@ -1,10 +1,12 @@
-#' Supported mvgam trend models
+#' Supported latent trend models in \pkg{mvgam}
 #' @importFrom utils tail
 #' @importFrom stats rnorm
 #' @details \code{mvgam} currently supports the following dynamic trend models:
 #'\itemize{
 #'   \item `None` (no latent trend component; i.e. the GAM component is all that contributes to the linear predictor,
 #'and the observation process is the only source of error; similarly to what is estimated by \code{\link[mgcv]{gam}})
+#'   \item `ZMVN()` (zero-mean correlated errors, useful for modelling time series where no
+#'   autoregressive terms are needed or for modelling data that are not sampled as time series)
 #'   \item `RW()`
 #'   \item `AR(p = 1, 2, or 3)`
 #'   \item `CAR(p = 1)`(continuous time autoregressive trends; only available in \code{Stan})
@@ -17,43 +19,12 @@
 #'measured in discrete, regularly spaced intervals (i.e. `c(1, 2, 3, ...)`). However you can
 #'use irregularly spaced intervals if using `trend_model = CAR(1)`, though note that any
 #'temporal intervals that are exactly `0` will be adjusted to a very small number
-#'(`1e-12`) to prevent sampling errors. For all trend types
-#'apart from `GP()`, `PW()`, and `CAR()`, moving average and/or correlated
+#'(`1e-12`) to prevent sampling errors. For all autoregressive trend types
+#'apart from `CAR()`, moving average and/or correlated
 #'process error terms can also be estimated (for example, `RW(cor = TRUE)` will set up a
-#'multivariate Random Walk if `data` contains `>1` series). Character strings can also be supplied
-#'instead of the various trend functions. The full list of possible models that are
-#'currently supported is:
-#'\itemize{
-#'\item 'RW'
-#'\item 'RWMA'
-#'\item 'RWcor'
-#'\item 'RWMAcor'
-#'\item 'AR1'
-#'\item 'AR1MA'
-#'\item 'AR1cor'
-#'\item 'AR1MAcor'
-#'\item 'AR2'
-#'\item 'AR2MA'
-#'\item 'AR2cor'
-#'\item 'AR2MAcor'
-#'\item 'AR3'
-#'\item 'AR3MA'
-#'\item 'AR3cor'
-#'\item 'AR3MAcor'
-#'\item 'CAR1'
-#'\item 'VAR'
-#'\item 'VARcor'
-#'\item 'VAR1' (same as 'VAR')
-#'\item 'VAR1cor' (same as 'VARcor')
-#'\item 'VARMA'
-#'\item 'VARMAcor'
-#'\item 'VARMA1,1cor'
-#'\item 'PWlinear'
-#'\item 'PWlogistic'
-#'\item 'GP'
-#'\item 'None'
-#'}
-#'
+#'multivariate Random Walk if `data` contains `>1` series). Hierarchical process error correlations
+#'can also be handled if the data contain relevant observation units that are nested into
+#'relevant grouping and subgrouping levels (i.e. using `AR(gr = region, subgr = species)`)
 #'
 #'Note that only `RW`, `AR1`, `AR2` and `AR3` are available if
 #'using `JAGS`. All trend models are supported if using `Stan`.
@@ -70,7 +41,7 @@
 #'by `Facebook`, where users can allow for changepoints to control the potential flexibility
 #'of the trend. See Taylor and Letham (2018) for details
 #'@seealso \code{\link{RW}}, \code{\link{AR}}, \code{\link{CAR}},
-#'\code{\link{VAR}}, \code{\link{PW}}, \code{\link{GP}}
+#'\code{\link{VAR}}, \code{\link{PW}}, \code{\link{GP}}, \code{\link{ZMVN}}
 #' @references Sarah E. Heaps (2022) Enforcing stationarity through the prior in Vector Autoregressions.
 #' Journal of Computational and Graphical Statistics. 32:1, 1-10.
 #'
@@ -86,30 +57,39 @@ trend_model_choices = function(){
   c("RW",
     "RWMA",
     "RWcor",
+    "RWhiercor",
     "RWMAcor",
     "GP",
     'AR1',
     'AR1MA',
     'AR1cor',
+    'AR1hiercor',
     'AR1MAcor',
     'AR2',
     'AR2MA',
     'AR2cor',
+    'AR2hiercor',
     'AR2MAcor',
     'AR3',
     'AR3MA',
     'AR3cor',
+    'AR3hiercor',
     'AR3MAcor',
     'CAR1',
     'VAR',
     'VARcor',
+    'VARhiercor',
     'VAR1',
     'VAR1cor',
+    'VAR1hiercor',
     'VARMA',
     'VARMAcor',
     'VARMA1,1cor',
     'PWlinear',
     'PWlogistic',
+    'ZMVN',
+    'ZMVNcor',
+    'ZMVNhiercor',
     'None')
 }
 
@@ -127,30 +107,40 @@ ma_cor_additions = function(trend_model){
 
   if(trend_model == 'AR3MA') trend_model <- 'AR3'
 
-  if(trend_model %in% c('RWcor', 'RWMAcor')){
+  if(trend_model %in% c('RWcor', 'RWhiercor', 'RWMAcor')){
     add_cor <- TRUE
     trend_model <- 'RW'
   }
 
-  if(trend_model %in% c('AR1cor', 'AR1MAcor')){
+  if(trend_model %in% c('ZMVNcor', 'ZMVNhiercor')){
+    add_cor <- TRUE
+    trend_model <- 'ZMVN'
+  }
+
+  if(trend_model %in% c('AR1cor', 'AR1hiercor', 'AR1MAcor')){
     add_cor <- TRUE
     trend_model <- 'AR1'
   }
 
-  if(trend_model %in% c('AR2cor', 'AR2MAcor')){
+  if(trend_model %in% c('AR2cor', 'AR2hiercor', 'AR2MAcor')){
     add_cor <- TRUE
     trend_model <- 'AR2'
   }
 
-  if(trend_model %in% c('AR3cor', 'AR3MAcor')){
+  if(trend_model %in% c('AR3cor', 'AR3hiercor', 'AR3MAcor')){
     add_cor <- TRUE
     trend_model <- 'AR3'
   }
 
   if(trend_model == 'VAR1') use_var1 <- TRUE
 
-  if(trend_model %in% c('VAR1cor', 'VARMA1,1cor')){
+  if(trend_model %in% c('VAR1cor', 'VAR1hiercor', 'VARMA1,1cor')){
     use_var1cor <- TRUE
+
+    if(trend_model == 'VAR1hiercor'){
+      add_cor <- TRUE
+    }
+
     trend_model <- 'VAR1'
   }
 
@@ -676,25 +666,31 @@ trend_par_names = function(trend_model,
                            drift = FALSE){
 
   # Check arguments
-  trend_model <- validate_trend_model(trend_model, drift = drift)
+  trend_model <- validate_trend_model(trend_model, drift = drift,
+                                      warn = FALSE)
 
   if(use_lv){
-    if(trend_model == 'RW'){
+    if(grepl('ZMVN', trend_model)){
       param <- c('trend', 'LV', 'penalty',
                  'lv_coefs', 'theta', 'Sigma', 'error')
     }
 
-    if(trend_model == 'AR1'){
+    if(grepl('RW', trend_model)){
+      param <- c('trend', 'LV', 'penalty',
+                 'lv_coefs', 'theta', 'Sigma', 'error')
+    }
+
+    if(grepl('AR1', trend_model)){
       param <- c('trend', 'ar1', 'LV', 'penalty',
                  'lv_coefs', 'theta', 'Sigma', 'error')
     }
 
-    if(trend_model == 'AR2'){
+    if(grepl('AR2', trend_model)){
       param <- c('trend', 'ar1', 'ar2', 'LV',
                  'penalty', 'lv_coefs', 'theta', 'Sigma', 'error')
     }
 
-    if(trend_model == 'AR3'){
+    if(grepl('AR3', trend_model)){
       param <- c('trend', 'ar1', 'ar2', 'ar3',
                  'LV', 'penalty', 'lv_coefs', 'theta',
                  'Sigma', 'error')
@@ -710,7 +706,7 @@ trend_par_names = function(trend_model,
                  'LV', 'lv_coefs', 'b_gp')
     }
 
-    if(trend_model == 'VAR1'){
+    if(grepl('VAR', trend_model)){
       param <- c('trend', 'A', 'Sigma',
                  'lv_coefs', 'LV', 'P_real', 'sigma',
                  'theta', 'error')
@@ -719,21 +715,25 @@ trend_par_names = function(trend_model,
   }
 
   if(!use_lv){
-    if(trend_model == 'RW'){
+    if(grepl('ZMVN', trend_model)){
       param <- c('trend', 'tau', 'sigma', 'theta', 'Sigma', 'error')
     }
 
-    if(trend_model == 'AR1'){
+    if(grepl('RW', trend_model)){
+      param <- c('trend', 'tau', 'sigma', 'theta', 'Sigma', 'error')
+    }
+
+    if(grepl('AR1', trend_model)){
       param <- c('trend', 'tau', 'sigma', 'ar1',
                  'theta', 'Sigma', 'error')
     }
 
-    if(trend_model == 'AR2'){
+    if(grepl('AR2', trend_model)){
       param <- c('trend', 'tau', 'sigma', 'ar1', 'ar2',
                  'theta', 'Sigma', 'error')
     }
 
-    if(trend_model == 'AR3'){
+    if(grepl('AR3', trend_model)){
       param <- c('trend', 'tau', 'sigma', 'ar1', 'ar2',
                  'ar3', 'theta', 'Sigma', 'error')
     }
@@ -747,7 +747,7 @@ trend_par_names = function(trend_model,
       param <- c('trend', 'alpha_gp', 'rho_gp', 'b_gp')
     }
 
-    if(trend_model == 'VAR1'){
+    if(grepl('VAR', trend_model)){
       param <- c('trend', 'A', 'Sigma', 'P_real',
                  'sigma', 'theta', 'error')
     }
@@ -764,6 +764,10 @@ trend_par_names = function(trend_model,
     }
   }
 
+  if(grepl('hiercor', trend_model)){
+    param <- c(param, 'alpha_cor')
+  }
+
   if(trend_model == 'None'){
     param <- NULL
   }
@@ -835,7 +839,7 @@ extract_trend_pars = function(object, keep_all_estimates = TRUE,
   # Latent trend loadings for dynamic factor models
   if(object$use_lv){
     if(attr(object$model_data, 'trend_model') %in% c('RW', 'AR1', 'AR2', 'AR3',
-                                                     'CAR1')){
+                                                     'CAR1', 'ZMVN')){
       # Just due to legacy reasons from working in JAGS, the simulation
       # functions use precision (tau) rather than SD (sigma)
       out$tau <- mcmc_chains(object$model_output, 'penalty')
@@ -1037,6 +1041,10 @@ extract_trend_pars = function(object, keep_all_estimates = TRUE,
 
   }
 
+  if(attr(object$model_data, 'trend_model') == 'ZMVN'){
+    out$ar1 <- rep(0, NROW(out$Sigma))
+  }
+
   # Return list of extracted posterior parameter samples
   out
 }
@@ -1191,7 +1199,11 @@ forecast_trend = function(trend_model, use_lv, trend_pars,
                        byrow = TRUE)
         ar1 <- rlang::missing_arg()
       } else if('ar1' %in% names(trend_pars)){
-        ar1 <- trend_pars$ar1
+        if(trend_pars$ar1[1] == 0){
+          ar1 <- rep(0, length(trend_pars$last_lvs))
+        } else {
+          ar1 <- trend_pars$ar1
+        }
         Amat <- rlang::missing_arg()
       } else {
         ar1 <- rep(1, length(trend_pars$last_lvs))
@@ -1331,7 +1343,11 @@ forecast_trend = function(trend_model, use_lv, trend_pars,
                        byrow = TRUE)
         ar1 <- rlang::missing_arg()
       } else if('ar1' %in% names(trend_pars)){
-        ar1 <- trend_pars$ar1
+        if(trend_pars$ar1[1] == 0){
+          ar1 <- rep(0, length(trend_pars$last_lvs))
+        } else {
+          ar1 <- trend_pars$ar1
+        }
         Amat <- rlang::missing_arg()
       } else {
         ar1 <- rep(1, length(trend_pars$last_lvs))
@@ -1437,11 +1453,11 @@ forecast_trend = function(trend_model, use_lv, trend_pars,
         changepoint_ts <- sort(c(trend_pars$changepoints, t_change_new))
 
         # Generate a trend draw
-        draw <- piecewise_linear(t = 1:max(time),
-                                 deltas = deltas,
-                                 k = trend_pars$k_trend[x],
-                                 m = trend_pars$m_trend[x],
-                                 changepoint_ts = changepoint_ts)
+        draw <- suppressWarnings(piecewise_linear(t = 1:max(time),
+                                                  deltas = deltas,
+                                                  k = trend_pars$k_trend[x],
+                                                  m = trend_pars$m_trend[x],
+                                                  changepoint_ts = changepoint_ts))
 
         # Keep only the forecast horizon estimates
         tail(draw, max(time) - min(time))
