@@ -1,8 +1,27 @@
 #'Argument validation functions
 #'@param data Data to be validated (list or dataframe)
 #'@noRd
-validate_series_time = function(data, name = 'data',
-                                trend_model){
+validate_series_time = function(data,
+                                name = 'data',
+                                trend_model,
+                                check_levels = TRUE,
+                                check_times = TRUE){
+
+  # First validation requires the full trend_model object
+  if(!inherits(trend_model, 'mvgam_trend')){
+    trend_model <- list(trend_model = trend_model,
+                        unit = 'time',
+                        gr = 'NA',
+                        subgr = 'series')
+  }
+
+  # Validate any grouping structure and update the data accordingly
+  data <- validate_series_groups(data = data,
+                                 trend_model = trend_model,
+                                 name = name)
+
+  # Now we only need the character trend_model string
+  trend_model <- trend_model$trend_model
 
   # Series label must be present as a factor and
   # must contain a time variable
@@ -60,6 +79,13 @@ validate_series_time = function(data, name = 'data',
     }
   }
 
+  # Add an identifier so post-processing functions know
+  # what the original supplied data ordering was; this is needed
+  # for ensuring that functions such as fitted() and
+  # residuals() return objects that match the original order that
+  # the user supplied, if no newdata are given
+  data$index..orig..order <- 1:length(data$time)
+
   # Add a new 'time' variable that will be useful for rearranging data for
   # modeling, in case 'time' is also supplied as a covariate or if this is
   # a continuous time model (CAR1)
@@ -80,39 +106,175 @@ validate_series_time = function(data, name = 'data',
       data$index..time..index <- times
   }
 
-  # Series factor must have all unique levels present
-  if(!all(levels(data$series) %in% unique(data$series))){
-    stop(paste0('Mismatch between factor levels of "series" and unique values of "series"',
-                '\n',
-                'Use\n  `setdiff(levels(data$series), unique(data$series))` \nand',
-                '\n',
-                '  `intersect(levels(data$series), unique(data$series))`\nfor guidance'),
-         call. = FALSE)
+  # Series factor must have all unique levels present if this is a
+  # forecast check
+  if(check_levels){
+    if(!all(levels(data$series) %in% unique(data$series))){
+      stop(paste0('Mismatch between factor levels of "series" and unique values of "series"',
+                  '\n',
+                  'Use\n  `setdiff(levels(data$series), unique(data$series))` \nand',
+                  '\n',
+                  '  `intersect(levels(data$series), unique(data$series))`\nfor guidance'),
+           call. = FALSE)
+    }
   }
 
   # Ensure each series has an observation, even if NA, for each
   # unique timepoint (only for trend models that require discrete time with
   # regularly spaced sampling intervals)
-  all_times_avail = function(time, min_time, max_time){
-    identical(as.numeric(sort(time)),
-              as.numeric(seq.int(from = min_time, to = max_time)))
-  }
-  min_time <- as.numeric(min(data$index..time..index))
-  max_time <- as.numeric(max(data$index..time..index))
-  data.frame(series = data$series,
-             time = data$index..time..index) %>%
-    dplyr::group_by(series) %>%
-    dplyr::summarise(all_there = all_times_avail(time,
-                                                 min_time,
-                                                 max_time)) -> checked_times
-  if(any(checked_times$all_there == FALSE)){
-    stop("One or more series in ", name, " is missing observations for one or more timepoints",
-         call. = FALSE)
+  if(check_times){
+    all_times_avail = function(time, min_time, max_time){
+      identical(as.numeric(sort(time)),
+                as.numeric(seq.int(from = min_time, to = max_time)))
+    }
+    min_time <- as.numeric(min(data$index..time..index))
+    max_time <- as.numeric(max(data$index..time..index))
+    data.frame(series = data$series,
+               time = data$index..time..index) %>%
+      dplyr::group_by(series) %>%
+      dplyr::summarise(all_there = all_times_avail(time,
+                                                   min_time,
+                                                   max_time)) -> checked_times
+    if(any(checked_times$all_there == FALSE)){
+      stop("One or more series in ", name, " is missing observations for one or more timepoints",
+           call. = FALSE)
+    }
   }
 
   return(data)
 }
 
+# Function to ensure units of analysis, groups and subgroups are arranged
+# and formatted properly for mvgam processing and modelling
+#'@noRd
+validate_series_groups = function(data, trend_model, name = 'data'){
+
+  # Checks only needed if trend_model isn't 'None'
+  if(trend_model$trend_model != 'None'){
+
+    # Check that unit and subgr exist in data and are the correct type
+    if(is.null(trend_model$gr)) trend_model$gr <- 'NA'
+    if(is.null(trend_model$unit)) trend_model$unit <- 'time'
+    if(is.null(trend_model$subgr)) trend_model$subgr <- 'series'
+
+    if(trend_model$gr == 'NA' &
+       trend_model$unit == 'time' &
+       trend_model$subgr == 'series'){
+
+      if(!'series' %in% names(data)){
+        data$series <- factor('series1')
+      }
+
+      if(!'time' %in% names(data)){
+        if(trend_model$trend_model %in% c('ZMVNcor',
+                                          'ZMVNhiercor')){
+          # Add a time indicator if missing
+          data.frame(series = data$series) %>%
+            dplyr::group_by(series) %>%
+            dplyr::mutate(time = dplyr::row_number()) %>%
+            dplyr::pull(time) -> times
+          data$time <- times
+        } else {
+          stop(name, " does not contain a 'time' variable",
+               call. = FALSE)
+        }
+      }
+    }
+    validate_var_exists(data = data,
+                        variable = trend_model$unit,
+                        type = 'num/int',
+                        name = name,
+                        trend_char = trend_model$trend_model)
+
+    validate_var_exists(data = data,
+                        variable = trend_model$subgr,
+                        type = 'factor',
+                        name = name,
+                        trend_char = trend_model$trend_model)
+
+    # If gr is supplied, check it exists and is the correct type
+    if(trend_model$gr != 'NA'){
+      validate_var_exists(data = data,
+                          variable = trend_model$gr,
+                          type = 'factor',
+                          name = name,
+                          trend_char = trend_model$trend_model)
+      if(trend_model$subgr == 'series'){
+        stop('argument "subgr" cannot be set to "series" if "gr" is also supplied',
+             call. = FALSE)
+      }
+
+      # Add necessary 'time' variable
+      gr_dat <- data.frame(time = data[[trend_model$unit]],
+                           gr = data[[trend_model$gr]],
+                           subgr = data[[trend_model$subgr]])
+
+      # Check that each level of gr contains all possible levels of subgr
+      gr_total_levels <- gr_dat %>%
+        dplyr::group_by(gr) %>%
+        dplyr::summarise(tot_subgrs = length(unique(subgr))) %>%
+        dplyr::ungroup() %>%
+        dplyr::pull(tot_subgrs)
+      if(length(gr_total_levels) > 1){
+        if(stats::var(gr_total_levels) != 0){
+          stop(paste0('Some levels of "', trend_model$gr, '" do not contain all\n',
+                      'unique levels of "', trend_model$subgr, '"',
+                      " in ", name),
+               call. = FALSE)
+        }
+      }
+
+      gr_dat %>%
+        dplyr::mutate(series = interaction(gr, subgr, drop = TRUE, sep = '_',
+                                           lex.order = TRUE)) -> gr_dat
+    } else {
+      gr_dat <- data.frame(time = data[[trend_model$unit]],
+                           subgr = data[[trend_model$subgr]]) %>%
+        dplyr::mutate(series = as.factor(subgr))
+    }
+
+    # Add the possibly new 'series' and 'time' variables to
+    # the data and return
+    data$series <- gr_dat$series
+    data$time <- gr_dat$time
+  }
+
+  return(data)
+}
+
+#'@noRd
+validate_var_exists = function(data,
+                               variable,
+                               type = 'factor',
+                               name = 'data',
+                               trend_char){
+  if(trend_char != 'None'){
+    if(!exists(variable, data)){
+      stop(paste0('Variable "', variable, '" not found in ', name),
+           call. = FALSE)
+    }
+
+    if(type == 'num/int'){
+      if(!is.numeric(data[[variable]])){
+        stop(paste0('Variable "', variable, '" must be either numeric or integer type'),
+             call. = FALSE)
+      }
+    }
+
+    if(type == 'factor'){
+      if(!is.factor(data[[variable]])){
+        stop(paste0('Variable "', variable, '" must be a factor type'),
+             call. = FALSE)
+      }
+    }
+  }
+}
+
+#'@noRd
+deparse_variable = function(...){
+  deparse0(substitute(...))
+}
+
 #'@noRd
 as_one_logical = function (x, allow_na = FALSE) {
   s <- substitute(x)
@@ -138,14 +300,19 @@ as_one_integer <- function(x, allow_na = FALSE) {
 }
 
 #'@noRd
-deparse0 = function (x, max_char = NULL, ...) {
-  out <- paste(deparse(x, ...), sep = "", collapse = "")
+deparse0 <- function (x, max_char = NULL, ...) {
+  out <- collapse(deparse(x, ...))
   if (isTRUE(max_char > 0)) {
     out <- substr(out, 1L, max_char)
   }
   out
 }
 
+#'@noRd
+collapse <- function(..., sep = "") {
+    paste(..., sep = sep, collapse = "")
+}
+
 #'@noRd
 validate_silent <- function(silent) {
   silent <- as_one_integer(silent)
@@ -198,8 +365,10 @@ validate_family = function(family, use_stan = TRUE){
          call. = FALSE)
 
   if(family$family %in% c('binomial', 'beta_binomial'))
-    rlang::warn(paste0("Binomial and Beta-binomial families require cbind(n_successes, n_trials)\nin the formula left-hand side. Do not use cbind(n_successes, n_failures)!"),
-                       .frequency = "once", .frequency_id = '1')
+    rlang::warn(paste0("Binomial and Beta-binomial families require cbind(n_successes, n_trials)\n",
+                       "in the formula left-hand side. Do not use cbind(n_successes, n_failures)!"),
+                .frequency = "once",
+                .frequency_id = 'cbind_binomials')
   return(family)
 }
 
@@ -247,14 +416,31 @@ validate_family_restrictions = function(response, family){
 }
 
 #'@noRd
-validate_trend_model = function(trend_model, drift = FALSE, noncentred = FALSE){
+validate_trend_model = function(trend_model,
+                                drift = FALSE,
+                                noncentred = FALSE,
+                                warn = TRUE){
   if(inherits(trend_model, 'mvgam_trend')){
     ma_term <- if(trend_model$ma){ 'MA' } else { NULL }
     cor_term <- if(trend_model$cor){ 'cor' } else { NULL }
+    if(is.null(trend_model$gr)) trend_model$gr <- 'NA'
+    if(trend_model$gr != 'NA'){
+      gr_term <- 'hier'
+    } else {
+      gr_term <- NULL
+    }
     trend_model <- paste0(trend_model$trend_model,
                           trend_model$p,
                           ma_term,
+                          gr_term,
                           cor_term)
+  } else {
+    if(trend_model != 'None' & warn){
+      rlang::warn(paste0("Supplying trend_model as a character string is deprecated\n",
+                         "Please use the dedicated functions (i.e. RW() or ZMVN()) instead"),
+                  .frequency = "once",
+                  .frequency_id = 'trend_characters')
+    }
   }
 
   trend_model <- match.arg(arg = trend_model,
@@ -266,11 +452,14 @@ validate_trend_model = function(trend_model, drift = FALSE, noncentred = FALSE){
   if(trend_model == 'VARcor'){
     trend_model <- 'VAR1cor'
   }
+  if(trend_model == 'VARhiercor'){
+    trend_model <- 'VAR1hiercor'
+  }
   if(trend_model %in% c('VARMA', 'VARMAcor')){
     trend_model <- 'VARMA1,1cor'
   }
 
-  if(!trend_model %in% c('None', 'RW','AR1', 'AR2', 'AR3', 'CAR1') & noncentred){
+  if(!trend_model %in% c('None', 'RW', 'AR1', 'AR2', 'AR3', 'CAR1') & noncentred){
     message('Non-centering of trends currently not available for this model')
   }
 
@@ -344,6 +533,32 @@ validate_trend_formula = function(formula){
 
 }
 
+#'@noRd
+validate_gr_subgr = function(gr, subgr, cor){
+  gr <- deparse0(substitute(gr))
+  subgr <- deparse0(substitute(subgr))
+
+  if(gr != 'NA'){
+    if(subgr == 'NA'){
+      stop('argument "subgr" must be supplied if "gr" is also supplied',
+           call. = FALSE)
+    }
+  }
+
+  if(subgr != 'NA'){
+    if(gr == 'NA'){
+      stop('argument "gr" must be supplied if "subgr" is also supplied',
+           call. = FALSE)
+    } else {
+      cor <- TRUE
+    }
+  }
+
+  list(.group = gr,
+       .subgroup = subgr,
+       .cor = cor)
+}
+
 #'@noRd
 validate_proportional = function(x){
   s <- substitute(x)
@@ -451,12 +666,6 @@ validate_trendmap = function(trend_map,
          call. = FALSE)
   }
 
-  # No point in trend mapping if trend model is 'None'
-  # if(trend_model == 'None'){
-  #   stop('cannot set up latent trends when "trend_model = None"',
-  #        call. = FALSE)
-  # }
-
   # trend_map must have an entry for each unique time series
   if(!all(sort(trend_map$series) == sort(unique(data_train$series)))){
     stop('Argument "trend_map" must have an entry for every unique time series in "data"',
@@ -567,12 +776,13 @@ validate_trend_restrictions = function(trend_model,
   if(!missing(trend_formula)){
     validate_trend_formula(trend_formula)
     if(missing(trend_map)){
-      trend_map <- data.frame(series = unique(data_train$series),
+      trend_map <- data.frame(series = factor(levels(data_train$series),
+                                              levels = levels(data_train$series)),
                               trend = 1:length(unique(data_train$series)))
     }
 
-    if(!trend_model %in% c('None', 'RW', 'AR1', 'AR2', 'AR3', 'VAR1', 'CAR1')){
-      stop('only None, RW, AR1, AR2, AR3, CAR1 and VAR trends currently supported for trend predictor models',
+    if(!trend_model %in% c('None', 'RW', 'AR1', 'AR2', 'AR3', 'VAR1', 'CAR1', 'ZMVN')){
+      stop('only None, ZMVN, RW, AR1, AR2, AR3, CAR1 and VAR trends currently supported for trend predictor models',
            call. = FALSE)
     }
   }
@@ -648,11 +858,43 @@ check_priorsim = function(prior_simulation, data_train, orig_y, formula){
 
 #'@noRd
 check_gp_terms = function(formula, data_train, family){
+
+  # Check for proper binomial specification
+  if(!missing(family)){
+    if(is.character(family)){
+      if(family == 'beta')
+        family <- betar()
+
+      family <- try(eval(parse(text = family)), silent = TRUE)
+
+      if(inherits(family, 'try-error'))
+        stop("family not recognized",
+             call. = FALSE)
+    }
+
+    if(is.function(family))
+      family <- family()
+
+    if(family$family %in% c('binomial', 'beta_binomial')){
+      # Check that response terms use the cbind() syntax
+      resp_terms <- as.character(terms(formula(formula))[[2]])
+      if(length(resp_terms) == 1){
+        stop('Binomial family requires cbind() syntax in the formula left-hand side',
+             call. = FALSE)
+      } else {
+        if(any(grepl('cbind', resp_terms))){
+        } else {
+          stop('Binomial family requires cbind() syntax in the formula left-hand side',
+               call. = FALSE)
+        }
+      }
+    }
+  }
+
   # Check for gp terms in the validated formula
   orig_formula <- gp_terms <- gp_details <- NULL
   if(any(grepl('gp(', attr(terms(formula), 'term.labels'), fixed = TRUE))){
 
-    # Check that there are no multidimensional gp terms
     formula <- interpret_mvgam(formula, N = max(data_train$time),
                                family = family)
     orig_formula <- formula
@@ -664,11 +906,11 @@ check_gp_terms = function(formula, data_train, family){
     gp_terms <- which_are_gp(formula)
 
     # Extract attributes
-    gp_details <- get_gp_attributes(formula)
+    gp_details <- get_gp_attributes(formula, data_train, family)
 
     # Replace with s() terms so the correct terms are included
     # in the model.frame
-    formula <- gp_to_s(formula)
+    formula <- gp_to_s(formula, data_train, family)
     if(!keep_intercept) formula <- update(formula, . ~ . - 1)
   }
 
@@ -806,3 +1048,15 @@ find_stan = function(){
     }
   }
 }
+
+#'@noRd
+as_one_character <- function(x, allow_na = FALSE) {
+  s <- substitute(x)
+  x <- as.character(x)
+  if (length(x) != 1L || anyNA(x) && !allow_na) {
+    s <- deparse0(s, max_char = 100L)
+    stop("Cannot coerce '", s, "' to a single character value.",
+         call. = FALSE)
+  }
+  x
+}
