@@ -27,16 +27,16 @@ test_that("series_to_mvgam working", {
 })
 
 test_that("stancode and standata working properly", {
-   simdat <- sim_mvgam()
-   mod <- mvgam(y ~ s(season) +
-                  s(time, by = series),
-                family = poisson(),
-                data = simdat$data_train,
-                run_model = FALSE)
+  simdat <- sim_mvgam()
+  mod <- mvgam(y ~ s(season) +
+                 s(time, by = series),
+               family = poisson(),
+               data = simdat$data_train,
+               run_model = FALSE)
 
-   expect_character(stancode(mod))
+  expect_character(stancode(mod))
 
-   expect_list(standata(mod))
+  expect_list(standata(mod))
 })
 
 # Skip remaining time-consuming tests on CRAN
@@ -55,7 +55,7 @@ test_that("add_residuals working properly", {
 test_that("mcmc diagnostics working properly", {
   expect_true(inherits(nuts_params(mvgam:::mvgam_example1),
                        'data.frame'))
-  expect_true(inherits(nuts_params(mvgam:::mvgam_example5),
+  expect_true(inherits(nuts_params(mvgam:::mvgam_example2),
                        'data.frame'))
 
   expect_true(inherits(rhat(mvgam:::mvgam_example1),
@@ -72,17 +72,17 @@ test_that("mcmc diagnostics working properly", {
 test_that("compute_edf working properly", {
   mod <- mvgam:::mvgam_example1
   expect_no_error(capture_output(mvgam:::compute_edf(mod$mgcv_model,
-                                   mod,
-                                   'rho',
-                                   'sigma_raw',
-                                   conservative = FALSE)))
+                                                     mod,
+                                                     'rho',
+                                                     'sigma_raw',
+                                                     conservative = FALSE)))
 
   mod <- mvgam:::mvgam_example4
   expect_no_error(capture_output(mvgam:::compute_edf(mod$trend_mgcv_model,
-                                      mod,
-                                      'rho_trend',
-                                      'sigma_raw_trend',
-                                      conservative = TRUE)))
+                                                     mod,
+                                                     'rho_trend',
+                                                     'sigma_raw_trend',
+                                                     conservative = TRUE)))
 })
 
 test_that("conditional_effects works properly", {
@@ -106,30 +106,29 @@ test_that("mcmc_plot works properly", {
                           variable = variables(mvgam:::mvgam_example1)$observation_betas[2:3, 1]))
   expect_error(mcmc_plot(mvgam:::mvgam_example1, type = "density"), "Invalid plot type")
   expect_ggplot(SW(mcmc_plot(mvgam:::mvgam_example2, type = "neff")))
-  expect_ggplot(mcmc_plot(mvgam:::mvgam_example3, type = "acf"))
   expect_silent(p <- mcmc_plot(mvgam:::mvgam_example3, type = "areas"))
   expect_error(mcmc_plot(mvgam:::mvgam_example3, type = "hex"),
                "Exactly 2 parameters must be selected")
   expect_ggplot(mcmc_plot(mvgam:::mvgam_example4))
-  expect_no_error(SW(pairs(mvgam:::mvgam_example1)))
+  expect_no_error(SW(pairs(mvgam:::mvgam_example2)))
   expect_no_error(SW(pairs(mvgam:::mvgam_example4,
-        variable = c('sigma'), regex = TRUE)))
+                           variable = c('sigma'), regex = TRUE)))
 })
 
 test_that("pp_check and ppc work properly", {
   expect_ggplot(SW(SM(pp_check(mvgam:::mvgam_example1))))
   expect_ggplot(SW(SM(pp_check(mvgam:::mvgam_example1,
-                         newdata = mvgam:::mvgam_example1$obs_data[1:10, ]))))
+                               newdata = mvgam:::mvgam_example1$obs_data[1:10, ]))))
   expect_ggplot(SW(SM(pp_check(mvgam:::mvgam_example2, "stat", ndraws = 5))))
   expect_ggplot(SW(SM(pp_check(mvgam:::mvgam_example3, "error_binned"))))
   pp <- SW(SM(pp_check(object = mvgam:::mvgam_example4,
-                 type = "ribbon_grouped",
-                 group = "series",
-                 x = "season")))
+                       type = "ribbon_grouped",
+                       group = "series",
+                       x = "season")))
   expect_ggplot(pp)
   pp <- SW(SM(pp_check(mvgam:::mvgam_example2, type = "violin_grouped",
-                 group = "season",
-                 newdata = mvgam:::mvgam_example2$obs_data[1:10, ])))
+                       group = "season",
+                       newdata = mvgam:::mvgam_example2$obs_data[1:10, ])))
   expect_ggplot(pp)
   expect_ggplot(SW(SM(pp_check(mvgam:::mvgam_example4, prefix = "ppd"))))
 
@@ -162,19 +161,21 @@ test_that("as.data.frame and friends have resonable outputs", {
   expect_s3_class(out, "data.frame")
   expect_equal(names(out), c("(Intercept)",
                              "seriesseries_2",
-                             "seriesseries_3"))
+                             "s(season).1",
+                             "s(season).2",
+                             "s(season).3"))
 
   out <- as.data.frame(mvgam:::mvgam_example4, variable = 'trend_params')
   expect_s3_class(out, "data.frame")
-  expect_equal(names(out)[1], "A[1,1]")
+  expect_equal(names(out)[1], "alpha_gp[1]")
 
   out <- as.data.frame(mvgam:::mvgam_example4, variable = 'obs_params')
   expect_s3_class(out, "data.frame")
-  expect_equal(names(out), c("sigma_obs[1]", "sigma_obs[2]", "sigma_obs[3]"))
+  expect_equal(names(out), c("sigma_obs[1]", "sigma_obs[2]"))
 
   out <- as.matrix(mvgam:::mvgam_example2, variable = 'obs_params')
   expect_true(inherits(out, "matrix"))
-  expect_equal(dimnames(out)[[2]], c("sigma_obs[1]", "sigma_obs[2]", "sigma_obs[3]"))
+  expect_equal(dimnames(out)[[2]], c("sigma_obs[1]", "sigma_obs[2]"))
 })
 
 test_that("coef has resonable outputs", {
@@ -182,15 +183,14 @@ test_that("coef has resonable outputs", {
   expect_equal(rownames(out), c("(Intercept)",
                                 "s(season).1",
                                 "s(season).2",
-                                "s(season).3",
-                                "s(season).4"))
-  expect_equal(dim(out), c(5, 5))
+                                "s(season).3"))
+  expect_equal(dim(out), c(4, 5))
 })
 
 test_that("logLik has reasonable ouputs", {
   liks <- logLik(mvgam:::mvgam_example4)
   expect_equal(dim(liks),
-               c(30, NROW(mvgam:::mvgam_example2$obs_data)))
+               c(5, NROW(mvgam:::mvgam_example2$obs_data)))
   # NAs in observations should propagate for likelihood calculations
   expect_true(all(is.na(liks[,which(is.na(mvgam:::mvgam_example2$obs_data$y))])))
 })
@@ -198,39 +198,39 @@ test_that("logLik has reasonable ouputs", {
 test_that("predict has reasonable outputs", {
   gaus_preds <- predict(mvgam:::mvgam_example4, type = 'link', summary = FALSE)
   expect_equal(dim(gaus_preds),
-               c(30, NROW(mvgam:::mvgam_example2$obs_data)))
+               c(5, NROW(mvgam:::mvgam_example2$obs_data)))
 
   gaus_preds <- predict(mvgam:::mvgam_example3, type = 'response', summary = FALSE)
   expect_equal(dim(gaus_preds),
-               c(30, NROW(mvgam:::mvgam_example3$obs_data)))
+               c(5, NROW(mvgam:::mvgam_example3$obs_data)))
 
   expect_error(predict(mvgam:::mvgam_example1, type = 'latent_N'),
                '"latent_N" type only available for N-mixture models',
                fixed = TRUE)
 
-  preds <- predict(mvgam:::mvgam_example5, type = 'terms')
+  preds <- predict(mvgam:::mvgam_example3, type = 'terms')
   expect_true(inherits(preds, 'list'))
   expect_true(all.equal(names(preds$obs_effects), c('fit', 'se.fit')))
   expect_true(is.null(preds$process_effects))
 
-  preds <- predict(mvgam:::mvgam_example4, type = 'terms')
+  preds <- predict(mvgam:::mvgam_example2, type = 'terms')
   expect_true(inherits(preds, 'list'))
-  expect_true(all.equal(names(preds$obs_effects), c('fit', 'se.fit')))
+  expect_true(length(preds$obs_effects) == 0)
   expect_true(!is.null(preds$process_effects))
 
   preds <- predict(mvgam:::mvgam_example4, type = 'terms',
                    summary = FALSE)
   expect_true(inherits(preds, 'list'))
   expect_true(is.matrix(preds$obs_effects[[1]]))
-  expect_true(dim(preds$obs_effects[[1]])[[2]] == NROW(mvgam:::mvgam_example4$obs_data))
-  expect_true(is.matrix(preds$process_effects[[1]]))
 })
 
 test_that("get_predict has reasonable outputs", {
   gaus_preds <- predict(mvgam:::mvgam_example1, type = 'link',
                         process_error = FALSE,
                         summary = FALSE)
-  meffects_preds <- get_predict(mvgam:::mvgam_example1, type = 'link')
+  meffects_preds <- get_predict(mvgam:::mvgam_example1,
+                                newdata = mvgam:::mvgam_example1$obs_data,
+                                type = 'link')
   expect_true(NROW(meffects_preds) == NCOL(gaus_preds))
   expect_true(identical(meffects_preds$estimate,
                         apply(gaus_preds, 2, median)))
@@ -243,27 +243,32 @@ test_that("hindcast has reasonable outputs", {
   expect_s3_class(hc, 'mvgam_forecast')
   expect_true(is.null(hc$forecasts))
   expect_equal(dim(hc$hindcasts[[1]]),
-               c(30, NROW(mvgam:::mvgam_example2$obs_data) /
+               c(5, NROW(mvgam:::mvgam_example2$obs_data) /
                    nlevels(mvgam:::mvgam_example2$obs_data$series)))
   expect_equal(hc$train_observations[[1]],
                mvgam:::mvgam_example2$obs_data$y[
                  which(mvgam:::mvgam_example2$obs_data$series == 'series_1')])
 })
 
-test_that("plot_mvgam_series reasonable outputs", {
+test_that("plot_mvgam_resids gives reasonable outputs", {
+  expect_ggplot(plot_mvgam_resids(mvgam:::mvgam_example1))
+  expect_ggplot(plot_mvgam_resids(mvgam:::mvgam_example2))
+})
+
+test_that("plot_mvgam_series gives reasonable outputs", {
   simdat <- sim_mvgam()
-  expect_no_error(plot_mvgam_series(data = simdat$data_train,
-                                    newdata = simdat$data_test))
-  expect_no_error(plot_mvgam_series(data = simdat$data_train,
-                                    newdata = simdat$data_test,
-                                    series = 'all'))
-  expect_no_error(plot_mvgam_series(data = simdat$data_train,
-                                    newdata = simdat$data_test,
-                                    lines = FALSE))
-  expect_no_error(plot_mvgam_series(data = simdat$data_train,
-                                    newdata = simdat$data_test,
-                                    lines = FALSE,
-                                    series = 'all'))
+  expect_ggplot(plot_mvgam_series(data = simdat$data_train,
+                                  newdata = simdat$data_test))
+  expect_ggplot(plot_mvgam_series(data = simdat$data_train,
+                                  newdata = simdat$data_test,
+                                  series = 'all'))
+  expect_ggplot(plot_mvgam_series(data = simdat$data_train,
+                                  newdata = simdat$data_test,
+                                  lines = FALSE))
+  expect_ggplot(plot_mvgam_series(data = simdat$data_train,
+                                  newdata = simdat$data_test,
+                                  lines = FALSE,
+                                  series = 'all'))
 
   # Should also work for list data
   dat_train <- list()
@@ -278,32 +283,38 @@ test_that("plot_mvgam_series reasonable outputs", {
   }
   names(dat_test) <- colnames(simdat$data_test)
 
-  expect_no_error(plot_mvgam_series(data = dat_train))
-  expect_no_error(plot_mvgam_series(data = dat_train,
-                                    newdata = dat_test))
-  expect_no_error(plot_mvgam_series(data = dat_train,
-                                    newdata = dat_test,
-                                    series = 'all'))
-  expect_no_error(plot_mvgam_series(data = dat_train,
-                                    newdata = dat_test,
-                                    lines = FALSE))
-  expect_no_error(plot_mvgam_series(data = dat_train,
-                                    newdata = dat_test,
-                                    lines = FALSE,
-                                    series = 'all'))
+  expect_ggplot(plot_mvgam_series(data = dat_train))
+  expect_ggplot(plot_mvgam_series(data = dat_train,
+                                  newdata = dat_test))
+  expect_ggplot(plot_mvgam_series(data = dat_train,
+                                  newdata = dat_test,
+                                  series = 'all'))
+  expect_ggplot(plot_mvgam_series(data = dat_train,
+                                  newdata = dat_test,
+                                  lines = FALSE))
+  expect_ggplot(plot_mvgam_series(data = dat_train,
+                                  newdata = dat_test,
+                                  lines = FALSE,
+                                  series = 'all'))
+
+  # And for mvgam objects
+  expect_ggplot(plot_mvgam_series(object = mvgam:::mvgam_example1,
+                                    series = 1))
+  expect_no_error(SW(plot(mvgam:::mvgam_example1, type = 'series')))
 })
 
 test_that("forecast and ensemble have reasonable outputs", {
   set.seed(1234)
   mvgam_examp_dat <- sim_mvgam(family = gaussian(),
                                T = 40,
-                               prop_missing = 0.1)
+                               prop_missing = 0.1,
+                               n_series = 2)
   newdat <- mvgam_examp_dat$data_test
   fc <- forecast(object = mvgam:::mvgam_example4,
                  newdata = newdat)
   expect_s3_class(fc, 'mvgam_forecast')
   expect_equal(dim(fc$forecasts[[1]]),
-               c(30, NROW(newdat) /
+               c(5, NROW(newdat) /
                    nlevels(newdat$series)))
   expect_equal(fc$test_observations[[1]],
                newdat$y[which(newdat$series == 'series_1')])
@@ -328,3 +339,42 @@ test_that("forecast and ensemble have reasonable outputs", {
   # ndraws must be positive integer
   expect_error(ensemble(fc, fc2, ndraws = 0))
 })
+
+test_that("ensemble gives equal pooling", {
+  set.seed(1234)
+  mvgam_examp_dat <- sim_mvgam(family = gaussian(),
+                               T = 40,
+                               prop_missing = 0.1,
+                               n_series = 2)
+  newdat <- mvgam_examp_dat$data_test
+  fc <- forecast(object = mvgam:::mvgam_example4,
+                 newdata = newdat)
+  fc2 <- forecast(object = mvgam:::mvgam_example3,
+                  newdata = newdat)
+
+  # Replace casts with dummy data
+  fc$hindcasts = lapply(fc$hindcasts,
+                        \(series_hcs) matrix(4, 1, ncol(series_hcs)))
+  fc2$hindcasts = lapply(fc2$hindcasts,
+                         \(series_hcs) matrix(5, 10, ncol(series_hcs)))
+  fc$forecasts = lapply(fc$forecasts,
+                        \(series_fcs) matrix(1, 1, ncol(series_fcs)))
+  fc2$forecasts = lapply(fc2$forecasts,
+                         \(series_fcs) matrix(2, 10, ncol(series_fcs)))
+
+  n_draws <- 500
+  fc_ens <- ensemble(fc, fc2, ndraws = n_draws)
+
+  # Expect that roughly 50% of hindcasts should be a 4 and 50% a 5
+  four_props <- unlist(lapply(seq_along(fc_ens$hindcasts), function(x){
+    length(which(fc_ens$hindcasts[[x]] == 4)) / length(fc_ens$hindcasts[[x]])
+  }), use.names = FALSE)
+  expect_equal(four_props, rep(0.5, 2),  tolerance = 0.03)
+
+  # Expect that roughly 50% of forecasts should be a 1 and 50% a 2
+  two_props <- unlist(lapply(seq_along(fc_ens$hindcasts), function(x){
+    length(which(fc_ens$forecasts[[x]] == 2)) / length(fc_ens$forecasts[[x]])
+  }), use.names = FALSE)
+  expect_equal(two_props, rep(0.5, 2), tolerance = 0.03)
+})
+
