@@ -16,7 +16,7 @@ skip_on_cran()
 
 test_that("get_mvgam_priors works for a variety of ma and cor trends", {
   priors <- get_mvgam_priors(y ~ s(season, k = 7),
-                             trend_model = 'RW',
+                             trend_model = RW(),
                              family = gaussian(),
                              data = mvgam:::mvgam_examp_dat$data_train)
   expect_true(inherits(priors, 'data.frame'))
@@ -65,7 +65,7 @@ test_that("get_mvgam_priors finds all classes for which priors can be specified"
 
 test_that("family must be correctly specified", {
   expect_error(get_mvgam_priors(y ~ s(season),
-                                trend_model = 'AR1',
+                                trend_model = AR(),
                                 data = beta_data$data_train,
                                 family = 'besta'),
                'family not recognized')
@@ -80,13 +80,13 @@ test_that("trend_model must be correctly specified", {
 
 test_that("response variable must follow family-specific restrictions", {
   expect_error(get_mvgam_priors(y ~ s(season),
-                            trend_model = 'AR1',
+                            trend_model = AR(),
                             data = gaus_data$data_train,
                             family = lognormal()),
                'Values <= 0 not allowed for lognormal responses')
 
   expect_error(get_mvgam_priors(y ~ s(season),
-                                trend_model = 'AR1',
+                                trend_model = AR(),
                                 data = gaus_data$data_train,
                                 family = poisson()),
                'Values < 0 not allowed for count family responses')
@@ -95,7 +95,7 @@ test_that("response variable must follow family-specific restrictions", {
 test_that("default intercept prior should match brms implementation", {
   simdat <- sim_mvgam(family = gaussian(), mu = 500)
   def_prior <- get_mvgam_priors(y ~ s(season),
-                                trend_model = 'AR1',
+                                trend_model = AR(),
                                 data = simdat$data_train,
                                 family = gaussian())$prior[1]
 
@@ -105,7 +105,7 @@ test_that("default intercept prior should match brms implementation", {
 
   # Now try Student
   def_prior <- get_mvgam_priors(y ~ s(season),
-                                trend_model = 'AR1',
+                                trend_model = AR(),
                                 data = simdat$data_train,
                                 family = student_t())$prior[1]
 
@@ -116,7 +116,7 @@ test_that("default intercept prior should match brms implementation", {
   # Now Poisson
   simdat <- sim_mvgam(family = poisson(), mu = 0)
   def_prior <- get_mvgam_priors(y ~ s(season),
-                                trend_model = 'AR1',
+                                trend_model = AR(),
                                 data = simdat$data_train,
                                 family = poisson())$prior[1]
 
@@ -127,7 +127,7 @@ test_that("default intercept prior should match brms implementation", {
   # Now Beta
   simdat <- sim_mvgam(family = betar(), mu = 0)
   def_prior <- get_mvgam_priors(y ~ s(season),
-                                trend_model = 'AR1',
+                                trend_model = AR(),
                                 data = simdat$data_train,
                                 family = betar())$prior[1]
 
@@ -138,7 +138,7 @@ test_that("default intercept prior should match brms implementation", {
   # Now Negative Binomial
   simdat <- sim_mvgam(family = nb(), mu = 0)
   def_prior <- get_mvgam_priors(y ~ s(season),
-                                trend_model = 'AR1',
+                                trend_model = AR(),
                                 data = simdat$data_train,
                                 family = nb())$prior[1]
 
@@ -150,12 +150,12 @@ test_that("default intercept prior should match brms implementation", {
 test_that("specified priors appear in the Stan code", {
 
   priors <- get_mvgam_priors(formula = y ~ s(season, bs = 'cc'),
-                             trend_model = 'GP',
+                             trend_model = GP(),
                              data = beta_data$data_train,
                              family = betar())
   priors$prior[3] <- "alpha_gp ~ normal(-1, 0.75);"
   stancode <- mvgam(y ~ s(season, bs = 'cc'),
-                    trend_model = 'GP',
+                    trend_model = GP(),
                     data = beta_data$data_train,
                     family = betar(),
                     priors = priors,
@@ -163,10 +163,10 @@ test_that("specified priors appear in the Stan code", {
   expect_true(expect_match2(gsub(' ', '', stancode),
                             'alpha_gp~normal(-1,0.75);'))
 
-  # Now the same using brms functionality; this time use a bound as well
-  priors <- prior(normal(-1, 0.75), class = alpha_gp, ub = 1)
+  # Now the same using brms functionality
+  priors <- prior(normal(-1, 0.75), class = alpha_gp)
   stancode <- mvgam(y ~ s(season, bs = 'cc'),
-                    trend_model = 'GP',
+                    trend_model = GP(),
                     data = beta_data$data_train,
                     family = betar(),
                     priors = priors,
@@ -174,7 +174,7 @@ test_that("specified priors appear in the Stan code", {
   expect_true(expect_match2(stancode,
                             'alpha_gp ~ normal(-1, 0.75);'))
   expect_true(expect_match2(gsub(' ', '', stancode),
-                            'vector<lower=0,upper=1>[n_series]alpha_gp;'))
+                            'vector<lower=0>[n_series]alpha_gp;'))
 })
 
 test_that("specified trend_formula priors appear in the Stan code", {
@@ -182,7 +182,7 @@ test_that("specified trend_formula priors appear in the Stan code", {
   priors <- get_mvgam_priors(formula = y ~ 1,
                              trend_formula = ~ s(season, bs = 'cc') +
                                year,
-                             trend_model = 'AR1',
+                             trend_model = AR(),
                              data = beta_data$data_train,
                              family = betar())
   priors$prior[5] <- "year_trend ~ uniform(-2, 1);"
@@ -190,7 +190,7 @@ test_that("specified trend_formula priors appear in the Stan code", {
   stancode <- mvgam(formula = y ~ 1,
                     trend_formula = ~ s(season, bs = 'cc') +
                       year,
-                    trend_model = 'AR1',
+                    trend_model = AR(),
                     data = beta_data$data_train,
                     family = betar(),
                     priors = priors,
@@ -203,12 +203,12 @@ test_that("specified trend_formula priors appear in the Stan code", {
 test_that("priors on parametric effects behave correctly", {
 
   priors <- get_mvgam_priors(formula = y ~ s(season, bs = 'cc'),
-                             trend_model = 'GP',
+                             trend_model = GP(),
                              data = beta_data$data_train,
                              family = betar())
   priors$prior[1] <- "(Intercept) ~ normal(-1, 0.75);"
   stancode <- mvgam(y ~ s(season, bs = 'cc'),
-                    trend_model = 'GP',
+                    trend_model = GP(),
                     data = beta_data$data_train,
                     family = betar(),
                     priors = priors,
@@ -219,7 +219,7 @@ test_that("priors on parametric effects behave correctly", {
   # Now the same using brms functionality
   priors <- prior(normal(-1, 0.75), class = Intercept)
   stancode <- mvgam(formula = y ~ s(season, bs = 'cc'),
-                    trend_model = 'GP',
+                    trend_model = GP(),
                     data = beta_data$data_train,
                     family = betar(),
                     priors = priors,
@@ -230,7 +230,7 @@ test_that("priors on parametric effects behave correctly", {
   # Bounds not allowed on parametric effect priors yet
   priors <- prior(normal(-1, 0.75), class = `Intercept`, lb = 0)
   expect_warning(mvgam(formula = y ~ s(season, bs = 'cc'),
-                       trend_model = 'GP',
+                       trend_model = GP(),
                        data = beta_data$data_train,
                        family = betar(),
                        priors = priors,
@@ -245,12 +245,25 @@ test_that("priors on gp() effects work properly", {
                   class = `alpha_gp(time):seriesseries_1`,
                   ub = 1),
               prior(normal(5, 1.3),
-                    class = `rho_gp_trend(season)`,
+                    class = `rho_gp_trend(season)[1]`,
                     ub = 50))
 
-  mod <- mvgam(formula = y ~ gp(time, by = series, scale = FALSE),
-               trend_formula = ~ gp(season, scale = FALSE),
-               trend_model = 'AR1',
+  expect_warning(mvgam(formula = y ~ gp(time, by = series, scale = FALSE, k = 10),
+                       trend_formula = ~ gp(season, scale = FALSE, k = 10),
+                       trend_model = AR(),
+                       data = dat$data_train,
+                       run_model = FALSE,
+                       priors = priors),
+                 'bounds cannot currently be changed for gp parameters')
+
+  priors <- c(prior(normal(0, 0.5),
+                    class = `alpha_gp(time):seriesseries_1`),
+              prior(normal(5, 1.3),
+                    class = `rho_gp_trend(season)[1]`))
+
+  mod <- mvgam(formula = y ~ gp(time, by = series, scale = FALSE, k = 10),
+               trend_formula = ~ gp(season, scale = FALSE, k = 10),
+               trend_model = AR(),
                data = dat$data_train,
                run_model = FALSE,
                priors = priors)
@@ -258,12 +271,33 @@ test_that("priors on gp() effects work properly", {
   # Observation model priors working
   expect_true(any(grepl('alpha_gp_time_byseriesseries_1~normal(0,0.5);',
                         gsub(' ', '', mod$model_file), fixed = TRUE)))
-  expect_true(any(grepl('real<lower=0,upper=1>alpha_gp_time_byseriesseries_1;',
-            gsub(' ', '', mod$model_file), fixed = TRUE)))
 
   # Process model priors working
-  expect_true(any(grepl('rho_gp_trend_season_~normal(5,1.3);',
+  expect_true(any(grepl('rho_gp_trend_season_[1]~normal(5,1.3);',
                         gsub(' ', '', mod$model_file), fixed = TRUE)))
-  expect_true(any(grepl('real<lower=0,upper=50>rho_gp_trend_season_;',
+
+  # A quick test of multidimensional gp priors
+  dat <- mgcv::gamSim(1, n = 30, scale = 2)
+
+  mod <- mvgam(y ~ gp(x1, x2,
+                       cov = "matern32",
+                       k = 10,
+                       iso = FALSE,
+                       scale = FALSE),
+                data = dat,
+                family = gaussian(),
+                priors = c(prior(exponential(2.5),
+                                 class = `alpha_gp(x1, x2)`),
+                           prior(normal(0.5, 1),
+                                 class = `rho_gp(x1, x2)[1][1]`),
+                           prior(normal(0.75, 2),
+                                 class = `rho_gp(x1, x2)[1][2]`)),
+                run_model = FALSE)
+
+  expect_true(any(grepl('alpha_gp_x1by_x2_~exponential(2.5);',
+                        gsub(' ', '', mod$model_file), fixed = TRUE)))
+  expect_true(any(grepl('rho_gp_x1by_x2_[1][1]~normal(0.5,1);',
+                        gsub(' ', '', mod$model_file), fixed = TRUE)))
+  expect_true(any(grepl('rho_gp_x1by_x2_[1][2]~normal(0.75,2);',
                         gsub(' ', '', mod$model_file), fixed = TRUE)))
 })
